//
//  NovationLogic.h
//  vst 2.4 examples
//
//  Created by ppad on 07/05/2015.
//
//

#ifndef __NovationLogic__
#define __NovationLogic__

#include <iostream>
#include <stdio.h>
#include "NovationDisplay.h"
#include "VSTXDrumKit.h"

namespace stk {

class NovationLogic
{
public:
    /*
     @nVoices = number of synth voices to control
  
     @nParams = number of parameters per synth voice

     @ptrNovDisplay = pointer to NovationDisplay

     @ptrDrumKit = pointer to VSTXDrumKit STK::Instrmnt

     @ptrNoteMidiChannel = pointer to midi channel value

     @voiceParamNamesAry = array of synth parameter names
     these are arranged in the actual indexed order as in
     the vstxvoice parameters
     */
    NovationLogic(
          NovationDisplay* ptrNovDisplay,
          VSTXDrumKit* ptrDrumKit,
          int* ptrNoteMidiChannel);
    ~NovationLogic();
  
    /* act on CCs sent from the control surface
     * e.g. when buttons are pressed/released
     */
    void midiCC(int ccNum, int ccVal);
  
private:
    bool buttonStates[127]; // raw novation button states
    int sliderStates[127];  // raw novation slider states
    int encoderStates[127]; // raw novation encoder states
    int potStates[127];     // raw novation pot states
    int potStatesPrev[127]; // previous raw novation pot states
    
    // calculated from raw button states CCs 88/89(left) and 90/91(right)
    int leftButtonIncDec;  // range +1, 0, -1
    int rightButtonIncDec; // range +1, 0, -1

    // ptr to NovationDisplay obj
    NovationDisplay *pNovDisp;
    
    // ptr to VSTXDrumKit Stk::Instrmnt object
    VSTXDrumKit *pDrumKit;

    // ptr to MIDI channel value
    int* pNoteMidiChannel;

    /*
     each "control slot" can be associated with a particular synth and parameter
     this array stores the associated assignments
     
     e.g.  controlAssignments[3].synth = 4 and
     controlAssignments[3].param = 12
     ...if control 3 is associated with synth 4 and parameter 12 (i.e. "OscBendA")
     
     see also:
     Assignments, paramNames
     */
    // TODO - maybe relabel these SLOTS not CONTROL-SLOTS?
    static const int kNumControlSlots = 16;
    struct controlAssignment
    {
        int synth;
        int param;
    };
    controlAssignment controlAssignments[kNumControlSlots];
    
    /*
     controlVals stores the associated values for each "control slot"
     
          if controlVals[3] = 102
          then whatever is assigned to slot 3 set to value 102
     
     e.g.  controlAssignments[3].synth = 4 and
           controlAssignments[3].param = 12
           then param 12 of synth 4 would be set to value 102
     */
    // TODO - maybe relabel these SLOTS not CONTROLS?
    int controlVals[kNumControlSlots];
    
    void initControlStates(void);
    void initControlAssignmentsAndVals(void);
    void refreshEntireDisplay(void);
    
    /* assign a slot to a param of a synth
    */
    void assign(int slot, int synth, int param);

    /* call upstream to tweak given \e slot to whatever
       is stored in controlVals[] for the given 
       \e synth and \e param
    */
    // NOTE: pDrumKit->setParam(int synth, int param, float value)
    void tweakSlot(int slot);


    /* midiCC() calls this 
       we make an internal copy of all ctl values
       which helps handleLogic() decide what to do
       with combinations or sequences of cc messages
    */
    void updateControlStates(int ccNum, int ccVal);

    /* midiCC() calls this
       we decide what to do with combinations or
       sequences of cc messages
    */
    void handleLogic(int ccNum, int ccVal);
    void handleLogicEnc(void); // encoders and enc-btns
    void handleLogicPot(void); // potentiometers and pot-btns
    void handleLogicSli(void);  // sliders or slid-btns

    //! display what the edge buttons do if they are held
    void handleEdgeBtnDisp(int ccNum, int ccVal);

    //! handle display of soundset and midi note channel number
    void handleSoundsetAndChannel(int ccNum, int ccVal);

    /* store and recall state on disk 
    */
    int soundsetFileNum; // sets (e.g. soundset 3 .dat)
    void saveSoundset(void);
    void loadSoundset(void);
    void saveDrumParamsFor(int drum);
    void loadDrumParamsFor(int drum);

    //TODO - move to tools
    int clipInt(int value, int min, int max); // hard limit
    int wrapInt(int value, int min, int max); // wrap around
};

} // namespace stk

#endif // define __NovationLogic__
