#ifndef STK_VSTXDRUMKIT_H
#define STK_VSTXDRUMKIT_H

#include "Instrmnt.h"
#include "VSTXDrum.h"

namespace stk {

class VSTXDrumKit : public Instrmnt
{
 public:

  // ctor
	//VSTXDrumKit( const int nDrums, const int* drumTriggerNotes );
  VSTXDrumKit();

	// dtor
	~VSTXDrumKit(void);

	//! Trigger sound for given \e noteNumber at \e amplitude
	void noteOn( StkFloat noteNumber, StkFloat amplitude );

	//! Stop a note with the given \e noteNumber
	void noteOff( StkFloat noteNumber );

  //! Return drum id number (zero-based) for the given noteNumber
  //! return "-1" if drum id not found
  int getDrumNumForNote( StkFloat noteNumber );

  //! get number of drums in the drumkit
  int getNDrums(void){ return nDrums; };

  //! get number of synthesis parameters for given \e drum
  int getNParamsFor( int drum );

	//! adjust synth param for given \e drum \e param
  //! \e value range is 0.0 to 1.0
	void setParam( int drum, int param, StkFloat value );

  //! inspect value for given \e drum \e param
  //! return value is in range 0.0. to 1.0
  StkFloat getParam( int drum, int param );

  //! get string param name for given \e drum \e param
  char* getParamName( int drum, int param );

	//! Compute and return one output sample - a mix of all drum sounds
	StkFloat tick( unsigned int channel = 0 );

	//! Fill a channel of the StkFrames object with computed outputs.
	StkFrames& tick( StkFrames& frames, unsigned int channel = 0 );

 private:
  int nDrums;
	VSTXDrum **drums; // array of pointers to VSTXDrum instrmnts
	int *drumTriggerNotes; // array of note numbers to trig each drum instrmnt

}; // class

inline StkFloat VSTXDrumKit :: tick( unsigned int )
{
  float sample = 0;
  for(int d=0; d < nDrums; d++) {
     sample += drums[d]->tick();
  }
  return sample;
}

inline StkFrames& VSTXDrumKit :: tick( StkFrames& frames, unsigned int channel )
{
	oStream_ << "VSTXDrumKit::tick(frames, chan)";
	handleError( StkError::STATUS );

  unsigned int nChannels = lastFrame_.channels();

#if defined(_STK_DEBUG_)
  if ( channel > frames.channels() - nChannels ) {
    oStream_ << "VSTXDrumKit::tick() chan and StkFrames args incompatible!";
    handleError( StkError::FUNCTION_ARGUMENT );
  }
#endif

  StkFloat *samples = &frames[channel];
  unsigned int j, hop = frames.channels() - nChannels;
  if ( nChannels == 1 ) {
    for ( unsigned int i=0; i<frames.frames(); i++, samples += hop ) {
      *samples++ = tick();
    }
  }
  else {
    for ( unsigned int i=0; i<frames.frames(); i++, samples += hop ) {
      *samples++ = tick();
      for ( j=1; j<nChannels; j++ )
        *samples++ = lastFrame_[j];
    }
  }

  return frames;
}

}// namespace stk

#endif /* STK_VSTXDRUMKIT_H */
