/*
   for xdrum (drum synthesiser with novation remote zero control panel)
   this program tests novation control logic - no synthesis takes place

   phill phelps zenpho@zenpho.co.uk 2015 may 31
*/
#include <iostream>
#include <cstdlib>
#include <RtMidi.h>

#include <NovationLogic.h>
#include <NovationDisplay.h>

struct TickData
{
  int things;
  bool isOn;
  bool isOff;
  char eightslots[8];

  TickData()
    : things(3), isOn(false), isOff(!isOn), eightslots() {}
};

// global
NovationLogic *logic = NULL;
NovationDisplay *disp = NULL;

void midiInCallback( double delta, 
      std::vector<unsigned char> *msg,
      void* userdata )
{
   TickData* data = (TickData*)userdata;
   std::cout << data->things << '\n';

   int size = msg->size();
   for(int i=0; i<size; i++)
   {
      unsigned char byte = msg->at(i);
      std::cout << (int)i << ": " << (int)byte << "\n";
   }

   if (size == 3 )
   {
     int type = msg->at(0) >> 4;
     //int chan = msg->at(0) & 0x0f;
     if (type == 11) { // CC
        logic->midiCC(msg->at(1), msg->at(2));

        if ( msg->at(2) == 127) data->things += 1;
     }
   }
}

RtMidiOut* midiOutput(std::string desiredPortName)
{
   RtMidiOut *port = NULL;

   try {
      port = new RtMidiOut();
   }
   catch ( RtMidiError &error ) {
      error.printMessage();
      return NULL;
   }

   int nPorts = port->getPortCount();
   std::cout << "\nThere are " << nPorts << " MIDI output destinations available.\n";
   std::string portName;
   for ( int i=0; i<nPorts; i++ ) {
      try {
         portName = port->getPortName(i);
      }
      catch ( RtMidiError &error ) {
         error.printMessage();
         return NULL;
      }
      std::cout << " #" << i+1 << ": " << portName << '\n';

      if (desiredPortName == portName)
      {
         try {
            std::cout << "Trying to open port " << (int)i << '\n';
            port->openPort(i);
            break;
         }
         catch ( RtMidiError &error ) {
            error.printMessage();
            return NULL;
         }
      }
      else {
         std::cout << "Skipping port " << (int)i << '\n';
      }
   }

   return port;
}

RtMidiIn* midiInput(std::string desiredPortName)
{
   RtMidiIn *port = NULL;

   try {
      port = new RtMidiIn();
   }
   catch ( RtMidiError &error ) {
      error.printMessage();
      return NULL;
   }

   int nPorts = port->getPortCount();
   std::cout << "\nThere are " << nPorts << " MIDI input sources available.\n";
   std::string portName;
   for ( int i=0; i<nPorts; i++ ) {
      try {
         portName = port->getPortName(i);
      }
      catch ( RtMidiError &error ) {
         error.printMessage();
         return NULL;
      }
      std::cout << " #" << i+1 << ": " << portName << '\n';

      if (desiredPortName == portName)
      {
         try {
            std::cout << "Trying to open port " << (int)i << '\n';
            port->openPort(i);
            break;
         }
         catch ( RtMidiError &error ) {
            error.printMessage();
            return NULL;
         }
      }
      else {
         std::cout << "Skipping port " << (int)i << '\n';
      }
   }

   return port;
}

int main(int argc, char* argv[])
{
  TickData data;
 
   const int nVoices = 6;
   const int drumTriggerNotes[nVoices] = {
      36, // C1
      38, // D1
      39, // D#1
      42, // F#1
      46, // A#1
      47  // B1
   };
   const int nParams = 17;
   const char* voiceParamNames[nParams] = {
        "ABend",
        "BendA",
        "BendD",
        
        "NAtk",
        "NBend",
        "NCrush",
        "NDcy",
        "NFreq",
        "NReso",
        "NVelo",
        "NLevel",
        
        "OAtk",
        "OBend",
        "OJitter",
        "ODcy",
        "OFreq",
        "OVelo"
    };

   // array of pointers to params for each voice in drums[]
   float *pParams[nVoices*nParams];
   /*for(int v=0; v<nVoices; v++) {
    for(int p=0; p<nParams; p++) {
      int idx = (v*nParams) + p;
      pParams[idx] = &(drums[v]->params[p]);
    }
   }*/

   RtMidiIn *ctlPort = NULL;
   RtMidiOut *dispPort = NULL;

   // try to open control port
   ctlPort = midiInput("ReMOTE ZeRO SL 20:1");

   // now display port
   dispPort = midiOutput("ReMOTE ZeRO SL 20:0");

   if ( ctlPort && dispPort) {
      disp = new NovationDisplay(dispPort);
      logic = new NovationLogic(nVoices, nParams, (char**)voiceParamNames, disp);

      ctlPort->setCallback( &midiInCallback, (void*)&data );

      char inchar;
      std::cin.get(inchar);
   }

   if (ctlPort) delete ctlPort;
   if (dispPort) delete dispPort;
   if (disp) delete disp;
   return 0;
}
