//
//  NovationDisplay.cpp
//  vst 2.4 examples
//
//  Created by ppad on 05/05/2015.
//
//

#include "NovationDisplay.h"

namespace stk {

NovationDisplay::NovationDisplay(RtMidiOut* dispPort)
{
   port = dispPort;
   reset();   
};

NovationDisplay::~NovationDisplay()
{
    
}

void NovationDisplay::reset(void)
{
   changeToReasonAutomapTemplate();
   clearLeftLCD();
   clearRightLCD();
   AllLightsOn();
}


void NovationDisplay::lcdClearText(int row, char* text)
{
   lcdClear(row);
   lcdText(row, text);
}

void NovationDisplay::lcdText(int row, char* text)
{
   const int pos = 0;
   lcdTextPos(row, pos, text);
}

void NovationDisplay::lcdTextPos(int row, int pos, char* text)
{
   std::vector<unsigned char> msg(12);

   unsigned char cmd[12] = {
      0xf0, 0x00, 0x20, 0x29,
      0x03, 0x03, 0x20, 0x02,
      0x01, 0x00, 0x02, 0x01
   };
   for(int i=0; i<12; i++){ msg.at(i) = cmd[i]; };
   msg.push_back(pos); 
   msg.push_back(row);
   msg.push_back(0x04); // text follows
   
   // 40 characters or EOS whichever comes first
   int len = 0;
   char *pPos = text;
   while( *pPos != '\0' && len++ < 40 ) {
      msg.push_back(*pPos);
      pPos++;
   }

   msg.push_back(0xf7);
   port->sendMessage(&msg);
}

void NovationDisplay::AllLightsOff(void)
{
    unsigned char msgA[3] = {
      0xB0, 0x60, 0x1f
    };
    std::vector<unsigned char> msgAV(msgA, msgA + sizeof(msgA) / sizeof(unsigned char) );
    port->sendMessage(&msgAV);

    unsigned char msgB[3] = {
      0xB0, 0x61, 0x00
    };
    std::vector<unsigned char> msgBV (msgB, msgB + sizeof(msgB) / sizeof(unsigned char) );
    port->sendMessage(&msgBV);
}

void NovationDisplay::AllLightsOn(void)
{
    unsigned char msgA[3] = {
      0xB0, 0x60, 0x1f
    };
    std::vector<unsigned char> msgAV(msgA, msgA + sizeof(msgA) / sizeof(unsigned char) );
    port->sendMessage(&msgAV);

    unsigned char msgB[3] = {
      0xB0, 0x61, 0x07
    };
    std::vector<unsigned char> msgBV (msgB, msgB + sizeof(msgB) / sizeof(unsigned char) );
    port->sendMessage(&msgBV);
}

void NovationDisplay::lcdClear(int id)
{
   std::vector<unsigned char> msg(12);
   unsigned char cmd[12] = {
      0xf0, 0x00, 0x20, 0x29,
      0x03, 0x03, 0x22, 0x03,
      0x01, 0x00, 0x02, 0x02
   };
   for(int i=0; i<=11; i++){ msg.at(i) = cmd[i]; };

   // which lcd id to clear
   // 4=top&btmleft, 5=top&btmright
   // 6=topleft, 7=btmleft, 8=topright, 9=btmright
   msg.push_back(id);

   msg.push_back(0xf7);
   port->sendMessage(&msg);
}

void NovationDisplay::clearRightLCD(void)
{
   lcdClear(5);
}

void NovationDisplay::clearLeftLCD(void)
{
   lcdClear(4);
}

void NovationDisplay::changeToReasonAutomapTemplate(void)
{
    unsigned char sysex[13] = {
      0xf0, 0x00, 0x20, 0x29,
      0x03, 0x03, 0x22, 0x03,
      0x01, 0x00, 0x01, 0x01,
      0xf7
    };
    std::vector<unsigned char> msgV (sysex, sysex + sizeof(sysex) / sizeof(unsigned char) );
    port->sendMessage(&msgV);
}

} //namespace stk
