//
//  novationLogic.cpp
//  vst 2.4 examples
//
//  Created by ppad on 07/05/2015.
//
//

#include "NovationLogic.h"
#include "string.h"

namespace stk {

NovationLogic::NovationLogic(
    NovationDisplay* ptrNovDisplay,
    VSTXDrumKit* ptrDrumKit,
    int* ptrNoteMidiChannel
    )
{
   // set member variables from ctor arguments
   pNovDisp = ptrNovDisplay;
   pDrumKit = ptrDrumKit;
   pNoteMidiChannel = ptrNoteMidiChannel;

   // init state machine
   initControlStates();
   initControlAssignmentsAndVals();

   //load default soundset and refresh display
   soundsetFileNum = 1;
   loadSoundset();
   refreshEntireDisplay();
}

NovationLogic::~NovationLogic()
{

}

//TODO: move to tools
inline int NovationLogic::clipInt(int value, int min, int max)
{
    if (value > max) value = max; // at max, clip at max
    if (value < min) value = min;
    return value;
}

//TODO: move to tools
inline int NovationLogic::wrapInt(int value, int min, int max)
{
    if (value > max) value = min; // at max, wraparound
    if (value < min) value = max;
    return value;
}

void NovationLogic::initControlStates()
{
    // buttons
    int i = 0; //24;
    do
    {
        buttonStates[i] = 0;
        i++;
    }while(i<=127); //39);

    // incdec
    leftButtonIncDec = 0;
    rightButtonIncDec = 0;

    // encoders
    i = 0;
    do
    {
        encoderStates[i] = 0;
        i++;
    }while(i<=7);

    // pots
    i = 0;
    do
    {
        potStates[i] = 0;
        potStatesPrev[i] = potStates[i];
        i++;
    }while(i<=7);

    // sliders
    do
    {
        sliderStates[i] = 0;
        i++;
    }while(i<=7);

    printf("initControlStates OK\n");
}

void NovationLogic::initControlAssignmentsAndVals(void)
{
    int i = 0;
    do
    {
        controlAssignments[i].synth = 0;
        controlAssignments[i].param = 0;
        controlVals[i] = 0;

        /*
        // this does not get the appropriate thing
        // if all are set to synth 0 param 0 (AllBendRange)
        // then all controls are set to the initial AllBendRange val
        int synth = controlAssignments[i].synth = 0;
        int param = controlAssignments[i].param = 0;
        StkFloat val = pDrumKit->getParam(synth, param);
        controlVals[i] = val * 127;
        */

        i++;
    }while(i<kNumControlSlots);
    
    printf("initControlAssignmentsAndVals OK\n");
}

/* refresh the entire display
*/
void NovationLogic::refreshEntireDisplay()
{
  // push and release each "modify this" button for each ctl
  for( int btnCCNum = 24; btnCCNum < 40; btnCCNum++ ) {
    midiCC(btnCCNum, 1);
    midiCC(btnCCNum, 0);
  }

  // quickly inc and dec the 'diskFileNum' preset control
  midiCC(90, 1);
  midiCC(90, 0);
  midiCC(91, 1);
  midiCC(91, 0);

  // press and release the "midi channel" modify button
  // but do not inc/dec
  midiCC(85, 1);
  midiCC(85, 0);
}

/* act on CCs sent from the control surface
 * e.g. when buttons are pressed/released
*/
void NovationLogic::midiCC(int ccNum, int ccVal)
{
    std::cout << (int)ccNum << ' ' << (int)ccVal << '\n';

    updateControlStates(ccNum, ccVal);
    handleLogic(ccNum, ccVal);
}

/*
 directly adjust controlAssignment[] array
 i.e. assign a slot to a param of a synth
 */
void NovationLogic::assign(int slot, int synth, int param)
{
    // value range limiting
    slot = clipInt(slot, 0, kNumControlSlots-1);

    int nDrums = pDrumKit->getNDrums();
    synth = clipInt(synth, 0, nDrums-1);

    int nParams = pDrumKit->getNParamsFor(synth);
    param = clipInt(param, 0, nParams-1);
                 
    controlAssignments[slot].synth = synth;
    controlAssignments[slot].param = param;   
}

// ///////////////////////////////////////////////////////////////////////

/* midiCC() calls this 
   we make an internal copy of all ctl values
   which helps handleLogic() decide what to do
   with combinations or sequences of cc messages
*/
void NovationLogic::updateControlStates(int ccNum, int ccVal)
{
    // buttons
    if (ccNum >= 24 && ccNum <= 39) // encoder and pot buttons
    {
        buttonStates[ccNum] = ccVal;
    }
    else if (ccNum >= 40 && ccNum <= 55) // under-slider buttons
    {
        buttonStates[ccNum] = ccVal;
    }
    else if (ccNum >= 80 && ccNum <= 87) // edge LED buttons
    {
        buttonStates[ccNum] = ccVal;
    }
    else if (ccNum >= 88 && ccNum <= 91 ) // LCD updown btns
    {
        // left LCD updown btns
        if ( ccNum == 88 && ccVal > 0 ) {
            leftButtonIncDec = 1;
        } else if ( ccNum == 89 && ccVal > 0 ) {
            leftButtonIncDec = -1;
        } else {
            leftButtonIncDec = 0;
        }
   
        // right LCD updown btns
        if ( ccNum == 90 && ccVal > 0 ) {
            rightButtonIncDec = 1;
        } else if ( ccNum == 91 && ccVal > 0 ) {
            rightButtonIncDec = -1;
        } else {
            rightButtonIncDec = 0;
        }
    }
    // encoders
    else if (ccNum >= 0 && ccNum <= 7 )
    {
        int diffVal = ccVal - 64;
        controlVals[ccNum] += diffVal;
        controlVals[ccNum] = clipInt(controlVals[ccNum], 0, 127);
        
        // store encoderStates[] as raw diffVal
        encoderStates[ccNum] = diffVal;
    } 
    // pots
    else if (ccNum >= 8 && ccNum <= 15 )
    {   
        // only update pot state if button pushed
        if ( buttonStates[ccNum + 24] ) 
        {
            // store new "previous" state
            potStatesPrev[ccNum -8] = potStates[ccNum -8];
            
            // update new "current" pot state
            potStates[ccNum -8] = ccVal;
        }

        controlVals[ccNum] = ccVal;
    }
    // sliders
    else if (ccNum >=16 && ccNum <= 23 )
    {
        // store values in sliderStates[] ary for convenience
        int sliderNum = ccNum - 16;
        sliderStates[sliderNum] = ccVal;
    }

    /* KLUGE! 
       encoders never explicitly transmit "idle", only explicit inc/dec
       so if last tweaked conrol was an encoder when a button is pushed
       then our handleLogic() thinks we're still moving the encoder
       kluge is to reset all encoder states to "idle" state (0)
       as soon as a non-encoder-control moves
    */
    if (ccNum > 7) // if not encoder
    {
        int i = 0;
        do {
            encoderStates[i] = 0;
            i++;
        }while(i<=7);
    }
}

/* call upstream to tweak given \e slot to whatever
   is stored in controlVals[]
*/
void NovationLogic::tweakSlot(int slot)
{
    int synth = controlAssignments[slot].synth;
    int paramIndex = controlAssignments[slot].param;
    int val = controlVals[slot];

    //std::cout << "tweakSlot " << slot << " " << val << '\n';
    
    // call upstream to adjust audio synthesis parameter
    pDrumKit->setParam(synth, paramIndex, val * ONE_OVER_128 );

    //char* paramString = paramNames[paramIndex];
    //var outList = ["vstparam", "synth" + synth, paramString, val];
    //post(outList + "\n");
    //outlet(0, outList);    
}

/* midiCC() calls this
   we decide what to do with combinations or
   sequences of cc messages
*/
void NovationLogic::handleLogic(int ccNum, int ccVal)
{
    handleLogicEnc();
    handleLogicPot();
    handleLogicSli(); // no slid logic yet just slid btns

    handleEdgeBtnDisp(ccNum, ccVal); // indicate what edge btns do
    handleSoundsetAndChannel(ccNum, ccVal); // display inc/dec midi channel

    // are we tweaking a pot/enc to adjust param
    int slot = clipInt(ccNum, 0, kNumControlSlots-1);
    if( ccNum <= 15 && buttonStates[slot + 24] == 0)
    {
        tweakSlot(slot);
    }

    // else are we adjusting volume with sliders
    else if ( ccNum >= 16 && ccNum <= 23 )
    {
        int sliderNum = ccNum - 16;
        int nDrums = pDrumKit->getNDrums();

        if ( sliderNum < nDrums ) {
          //pDrumKit->setVolume(sliderNum, ccVal * ONE_OVER_128 );
        }
    }
   
    // else is "record" button requesting LOAD state
    else if( ccNum == 76 && ccVal == 1)
    {
        // start sysexdump by "record" button (cc 76 0x4c)
        //TODO - call upstream to invoke sysex dump

        // write state to disk
        saveSoundset();
    }

    // else is "play" button requesting LOAD state
    else if ( ccNum == 75 && ccVal == 1 )
    {
        // recall state from disk
        loadSoundset();
        refreshEntireDisplay();
    }

}

/* save a given drum param set to disk
   useful for copy-pasting drum sounds between
   soundsets
*/
void NovationLogic::saveDrumParamsFor(int drum)
{
  int nDrums = pDrumKit->getNDrums();
  if ( drum > nDrums-1 || drum < 0 ) return;

  FILE* fp = fopen("tempSound.dat", "wb");
  if ( fp ) {
    
    int nParams = pDrumKit->getNParamsFor(drum);
    for(int param=0; param<nParams; param++) {
      StkFloat value = 0.0;
      value = pDrumKit->getParam(drum, param);
      fwrite(&value, sizeof(value), 1, fp);
    }

    fclose(fp);   
  }
}

/* load a given drum param set from disk
   useful for copy-pasting drum sounds between
   soundsets
*/
void NovationLogic::loadDrumParamsFor(int drum)
{
  std::cout << "load " << (int)drum << '\n';

  int nDrums = pDrumKit->getNDrums();
  if ( drum > nDrums-1 || drum < 0 ) return;

  FILE* fp = fopen("tempSound.dat", "rb");
  if ( fp ) {
 
    int nParams = pDrumKit->getNParamsFor(drum);
    for(int param=0; param<nParams; param++) {
      StkFloat value = 0.0;
      fread(&value, sizeof(value), 1, fp);
      pDrumKit->setParam(drum, param, value);
    }

    fclose(fp);   
  }
}

/* save a complete sound set file to disk
   whose name includes diskFileNum e.g. config3.dat
   and whose content includes all slot configuration
   and all synthesis params for the entire drumkit
*/
void NovationLogic::saveSoundset(void)
{
  FILE* fp;
  char *filename;
  asprintf(&filename, "soundset%3d.dat", soundsetFileNum);

  fp = fopen(filename, "wb");
  free(filename);

  if ( fp ) {
    // array of controlAssignments 
    fwrite(controlAssignments, 
           sizeof(*controlAssignments), 
           kNumControlSlots, 
           fp);

    // drumkit synthesis parameters
    // iterate over each parameter of each drum
    int nDrums = pDrumKit->getNDrums();
    for(int drum=0; drum<nDrums; drum++) {
      int nParams = pDrumKit->getNParamsFor(drum);

      StkFloat *pDrumParams = (StkFloat*) calloc(nParams, sizeof(StkFloat));
      StkFloat *pParam = pDrumParams;

      for(int param=0; param<nParams; param++) {
        *pParam = pDrumKit->getParam(drum, param);
        pParam++;
      }

      fwrite(pDrumParams, sizeof(*pDrumParams), nParams, fp);
      free(pDrumParams);  
    }   
    fclose(fp);
  }
}

/* load a complete sound set file from disk
   whose name includes diskFileNum e.g. config3.dat
   and whose content includes all slot configuration
   and all synthesis params for the entire drumkit
*/
void NovationLogic::loadSoundset(void)
{
  FILE* fp;
  char *filename;
  asprintf(&filename, "soundset%3d.dat", soundsetFileNum);
 
  fp = fopen(filename, "rb");
  free(filename);

  if ( fp != NULL ) {
    //array of controlAssignments 
    fread(controlAssignments, 
           sizeof(*controlAssignments), 
           kNumControlSlots, 
           fp);

    // drumkit synthesis parameters
    // iterate over each parameter of each drum
    int nDrums = pDrumKit->getNDrums();
    for(int drum=0; drum<nDrums; drum++) {
      
      int nParams = pDrumKit->getNParamsFor(drum);
      for(int param=0; param<nParams; param++) {
        StkFloat value = 0.0;
        fread(&value, sizeof(value), 1, fp);
        pDrumKit->setParam(drum, param, value);
      }

    }   
    fclose(fp);
  }
}

//! handle adjusting soundset and note midi channel value
void NovationLogic::handleSoundsetAndChannel(int ccNum, int ccVal)
{
    int chanlDispPos = 5;
    int soundetDispPos = 37;

    // are we about to modify the note midi channel - edge btn 85
    if ( buttonStates[85] != 0 )
    {
        if ( rightButtonIncDec != 0 )
        {
          *pNoteMidiChannel += rightButtonIncDec;
          *pNoteMidiChannel = wrapInt( *pNoteMidiChannel, 0, 15 );
        }

        // only display the number here
        // the "Chanl" prompt is displayed by handleEdgeBtnDisp()
        char *dispNum;
        asprintf(&dispNum, "%02d", *pNoteMidiChannel);

   
        pNovDisp->lcdTextPos(2, chanlDispPos, dispNum);
        free(dispNum);
    }
    // is the note midi chanl modify edgebtn 85 just being released?
    else if ( !ccVal && !buttonStates[85] ) 
    {
        // hide the channel number
        // the "Chanl" prompt is displayed by handleEdgeBtnDisp()
        char *dispNum;
        asprintf(&dispNum, "  ");

        pNovDisp->lcdTextPos(2, chanlDispPos, dispNum);
        free(dispNum);
    }

    // else are we inc/dec the settings file counter?
    else if ( !buttonStates[85] && rightButtonIncDec )
    {
        soundsetFileNum += rightButtonIncDec;
        soundsetFileNum = clipInt(soundsetFileNum, 0, 9);

        char *dispNum;
        asprintf(&dispNum, "Soundset%1d", soundsetFileNum);
        
        pNovDisp->lcdTextPos(2, soundsetDispPos, dispNum);
        free(dispNum);
    }

}

//! handle logic related to encoders (and encoder buttons)
void NovationLogic::handleLogicEnc()
{
    int slot = 0;
    do
    {
        int textPos = slot * 9;
        // is user holding down button to modify this slot?
        if( buttonStates[slot + 24] )
        {
            // is user requesting we RANDOMISE this slot value?
	          // NOTE for randomising entire drum sound see handleSli()
            if ( buttonStates[84] )
            {
              StkFloat rVal = rand() / (RAND_MAX + 1.0); // range 0.0 to 1.0
              controlVals[slot] = (int)( rVal * 127 );
              tweakSlot(slot);
            }
      
            // compute potential change to the SYNTH this slot controls
            int synth = controlAssignments[slot].synth;
            synth += leftButtonIncDec;
            int nDrums = pDrumKit->getNDrums();  // wrap around
            synth = wrapInt(synth, 0, nDrums-1 );

            // compute potential change to the PARAM this slot controls
            int paramIndex = controlAssignments[slot].param;
            paramIndex += encoderStates[slot];
            encoderStates[slot] = 0; // must force "idle" as encs only inc/dec
            int nParams = pDrumKit->getNParamsFor(synth); // wrap around
            paramIndex = wrapInt(paramIndex, 0, nParams-1);

            // if the controlAssignment[] is outdated then update it
            // we need this check because otherwise infinite recursion would
            // be set up when refreshEntireDisplay() calls midiCC() to
            // push buttons that cause handleLogic() to call this fn
            if ( controlAssignments[slot].synth != synth || 
                 controlAssignments[slot].param != paramIndex ) {
               
                  // change the assignment now - and save soundset
                  assign(slot, synth, paramIndex);
                  saveSoundset();
            }         

            //prepare to display
            char* paramName = pDrumKit->getParamName(synth, paramIndex);
            char *dispParam;
            asprintf(&dispParam, "%-7s", paramName);

            char *dispSynth;
            asprintf(&dispSynth, "%1d", synth);

            // set display
            pNovDisp->lcdTextPos(1, textPos+1, dispParam);
            pNovDisp->lcdTextPos(1, textPos, dispSynth);

            free(dispSynth);
            free(dispParam);

        }// btn[slot+24]

        slot++;
    } while(slot < 8); // TODO slot < kNumEncoders
}

//! handle logic related to pots (and pot buttons)
void NovationLogic::handleLogicPot(void)
{
    int slot = 8;
    do
    {
        int potNum = slot - 8;
        int textPos = potNum * 9;

        // is user holding down button to modify this slot?
        if( buttonStates[potNum + 32] )
        {
            // is user requesting we RANDOMISE this slot VALUE?
	          // NOTE for randomising entire drum sound see handleSli()
            if ( buttonStates[84] )
            {
              StkFloat rVal = rand() / (RAND_MAX + 1.0); // range 0.0 to 1.0
              controlVals[slot] = (int)( rVal * 127 );
              tweakSlot(slot);
            }

            // compute potential change to the SYNTH this slot controls
            int synth = controlAssignments[slot].synth;
            synth += leftButtonIncDec;
            int nDrums = pDrumKit->getNDrums();  // wrap around
            synth = wrapInt(synth, 0, nDrums-1 );
            
            // compute potential change to the PARAM this slot controls
            // only update if pot differs compared to previous state
            int paramIndex = controlAssignments[slot].param;
            if ( potStatesPrev[potNum] != potStates[potNum] )
            {
                // pot range 0-127 scaled across possible params to control
                int nParams = pDrumKit->getNParamsFor(synth);
                paramIndex = (potStates[potNum] / 127.0) * (nParams-1);           
            }

            // if the controlAssignment[] is outdated then update it
            // we need this check because otherwise infinite recursion would
            // be set up when refreshEntireDisplay() calls midiCC() to
            // push buttons that cause handleLogic() to call this fn
            if ( controlAssignments[slot].synth != synth || 
                 controlAssignments[slot].param != paramIndex ) {
               
                  // change the assignment now - and save soundset
                  assign(slot, synth, paramIndex);
                  saveSoundset();
            }
            
            // prepare to display
            char *paramName = pDrumKit->getParamName(synth, paramIndex);
            char *dispParam;
            asprintf(&dispParam, "%-7s", paramName);

            char *dispSynth;
            asprintf(&dispSynth, "%1d", synth);

            // set display
            pNovDisp->lcdTextPos(3, textPos+1, dispParam);
            pNovDisp->lcdTextPos(3, textPos, dispSynth);

            free(dispSynth);
            free(dispParam);
        }
        
        slot++;
    } while(slot < 16);  // TODO slot < kNumPots + 8
}

//! handle logic related to sliders (and slider buttons)
void NovationLogic::handleLogicSli()
{

  // is user reqing MODIFY of DRUMS by psh btns under sliders?
  for( int btn=40; btn<=47; btn++ ) {
    int drum = btn - 40; // which drum this btn refers to

    // COPY drum sound -- hold btn 86
    if ( buttonStates[86] && buttonStates[btn] ) {
      saveDrumParamsFor(drum); // save to "copybuffer" on disk
      break; // ignore any more pushed buttons
    }

    // PASTE drum sound -- hold btn 87
    if ( buttonStates[87] && buttonStates[btn] ) {
      loadDrumParamsFor(drum); 
      break; 
    }

    // RANDOMISE drum sound -- hold btn 84
    // note randomising slots also possible see handleEnc(), handlePot()
    if ( buttonStates[84] && buttonStates[btn] )
    {
      int nParams = pDrumKit->getNParamsFor(drum);
      for(int param=0; param<nParams; param++) {
        StkFloat rVal = rand() / (RAND_MAX + 1.0); // range 0.0 to 1.0
        pDrumKit->setParam(drum, param, rVal);
      }
    }

    // TODO - MUTE drum sound by pushing normally?
    if ( buttonStates[btn] )
    {

    }
  }

  // TODO - SOLO drum sound by psh 2nd row of btns?
  //for( int btn=48; btn<=55; btn++ ) {
  //  
  //}
}

//! indicate what the edge buttons do
void NovationLogic::handleEdgeBtnDisp(int ccNum, int ccVal)
{
  // display "Rand" "Chanl" "Copy" "Paste"
  char *dispFunc = NULL;

  if ( buttonStates[84] ) {
      asprintf(&dispFunc, "Rand ");
  }
  if ( buttonStates[85] ) {
      asprintf(&dispFunc, "Chanl");
  }
  else if ( buttonStates[86] ) {
      asprintf(&dispFunc, "Copy ");
  }
  else if ( buttonStates[87] ) {
      asprintf(&dispFunc, "Paste");
  }
  // is this a button being released and this is the last one?
  else if ( !ccVal &&
            !buttonStates[84] &&
            !buttonStates[85] &&
            !buttonStates[86] &&
            !buttonStates[87] ) {
      asprintf(&dispFunc, "     ");
  }

  if (dispFunc) {
      pNovDisp->lcdTextPos(2, 0, dispFunc); // set display
      
      free(dispFunc);
  }
}

} // namespace stk

