/*
 * VSTXDrumKit.cpp
 *
 *  Created on: 2 May 2015
 *      Author: zenpho@zenpho.co.uk
 */

#include "VSTXDrumKit.h"

namespace stk {

VSTXDrumKit::VSTXDrumKit()
{
  // set number of drums and assign trigger notes
  nDrums = 5;
  drumTriggerNotes = (int *) calloc( nDrums, sizeof(drumTriggerNotes) );
  drumTriggerNotes[0] = 36;
  drumTriggerNotes[1] = 38;
  drumTriggerNotes[2] = 39;
  drumTriggerNotes[3] = 42;
  drumTriggerNotes[4] = 46;

  // construct drum instruments
  drums = (VSTXDrum **) calloc( nDrums, sizeof(VSTXDrum *) );
  for (int i=0; i<nDrums; i++ )
    drums[i] = new VSTXDrum;
}

VSTXDrumKit::~VSTXDrumKit(void)
{
	for (int i=0; i<nDrums; i++ )
		delete drums[i];
  free(drums); 

  free(drumTriggerNotes);
}

//! Return drum id number (zero-based) for the given noteNumber
//! return "-1" if drum id not found
int VSTXDrumKit::getDrumNumForNote( StkFloat noteNumber )
{
  for(int i=0; i<nDrums; i++) {
    if ((int)noteNumber == drumTriggerNotes[i]) {
     return i;  // found it!
    }
  }
  return -1; // we didn't find it!
}

//! Trigger sound for given \e noteNumber at \e amplitude
void VSTXDrumKit::noteOn( StkFloat noteNumber, StkFloat amplitude )
{
  // iterate over drums in the kit
  for(int i=0; i<nDrums; i++) {
    if ((int)noteNumber == drumTriggerNotes[i]) {
     drums[i]->noteOn(noteNumber, amplitude);
    }
  }
}

//! Stop sound for \e noteNumber
void VSTXDrumKit::noteOff( StkFloat noteNumber )
{
  // iterate over drums in the kit
  for(int i=0; i<nDrums; i++) {
    if ((int)noteNumber == drumTriggerNotes[i] ) {
      drums[i]->noteOff(0.0);
    }
  }
}

//! get number of parameters for given \e drum
// TODO - maybe add vol, pan ctls here
int VSTXDrumKit::getNParamsFor( int drum )
{
  return drums[drum]->getNumControls();
}

//! adjust synth param for given \e drum \e param. \e value range 0 to 1.0
// TODO - maybe add vol, pan ctls here
void VSTXDrumKit::setParam( int drum, int param, StkFloat value ) 
{
  drums[drum]->controlChange(param, value);
}

//! inspect value for given \e drum \e param
// TODO - maybe add vol, pan ctls here
StkFloat VSTXDrumKit::getParam( int drum, int param )
{
  return drums[drum]->controlInspect(param);
}

//! ask for a string name of the given \e drum \e param
// TODO - maybe add vol, pan ctls here
char* VSTXDrumKit::getParamName( int drum, int param )
{
  return drums[drum]->getParamName(param);
}

} // namespace stk
