/*! \class VSTXResoFilter
    \brief dspwiki lowpass resonant biquad

    http://dspwiki.com/index.php?title=Lowpass_Resonant_Biquad_Filter
*/

#include "VSTXResoFilter.h"

// require M_PI and tan()
#include <math.h>

void VSTXResoFilter::filterConfig(float cutoffHz, float reso)
{
    if(cutoffHz < 19.0)
        cutoffHz = 19.0;

    //fCutoff = pow((float)1000.0,(float)cutoff)*20;
    fCutoff = cutoffHz;
    fResonance = 1 - reso;

    c  = 1.0 / tan(M_PI*fCutoff/fSampleRate);
    a1 = 1.0 / (1.0 + fResonance * c + c * c);
    a2 = 2 * a1;
    a3 = a1;
    b1 = 2.0 * (1.0 - c * c) * a1;
    b2 = ( 1.0 - fResonance * c + c * c) * a1;
}

float VSTXResoFilter::filter( float sval )
{
    float outVal;

    // rotate input buffer
    bufferIn[1]=bufferIn[0];
    bufferIn[0]=sval;

    outVal  = a1 * sval
            + a2 * bufferIn[0]
            + a3 * bufferIn[1]
            - b1 * bufferOut[0]
            - b2 * bufferOut[1];

    // rotate output buffer
    bufferOut[1]=bufferOut[0];
    bufferOut[0]=outVal;

    return (float)outVal;
}

VSTXResoFilter::VSTXResoFilter(float sampleRate)
{
    fSampleRate = sampleRate;
    for(long i = 0; i < 2; i++) {
            bufferIn[i] = bufferOut[i] = 0.0;
    }
}
